package ru.nsu.sberlab.api.impl;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import ru.nsu.sberlab.model.dto.user.LoginDto;
import ru.nsu.sberlab.model.dto.user.RefreshTokenDto;
import ru.nsu.sberlab.model.dto.user.UserRegistrationDto;
import ru.nsu.sberlab.security.SecurityService;

@RestController
@RequiredArgsConstructor
@Tag(name = "security", description = "API для работы с аутентификацией и регистрации пользователя")
@RequestMapping("api/v1/security")
@CrossOrigin(origins = "http://localhost:3000")
public class SecurityController {
    private final SecurityService securityService;

    @Operation(summary = "Регистрация пользователя")
    @PostMapping("/registration")
    public ResponseEntity<?> register(@RequestBody @Validated UserRegistrationDto registrationDto) {
        return ResponseEntity.ok(securityService.register(registrationDto));
    }

    @Operation(summary = "Аутентификация пользователя")
    @PostMapping("/login")
    public ResponseEntity<?> login(@RequestBody @Validated LoginDto loginDto) {
        return ResponseEntity.ok(securityService.authenticate(loginDto));
    }

    @Operation(summary = "Обновление access и refresh токенов")
    @GetMapping("/token/refresh")
    public ResponseEntity<?> refreshToken(@RequestBody @Validated RefreshTokenDto refreshTokenDto) {
        return ResponseEntity.ok(securityService.refreshToken(refreshTokenDto));
    }
}
